package GCImport::GCImportGCstar;

###################################################
#
#  Copyright 2005-2006 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use GCImport::GCImportBase;

{
    package GCImport::GCImporterGCstar;
    use base qw(GCImport::GCImportBaseClass);

    use File::Copy;
    
    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);
        return $self;
    }

    sub getName
    {
        return "GCstar (.gcs)";
    }
    
    sub getFilePatterns
    {
       return (['GCstar (.gcs)', '*.gcs']);
    }
    
    sub getModelName
    {
        my $self = shift;
        
        return $self->{extractedModel};
    }

    sub getOptions
    {
        my $self = shift;
        return [
        {
            name => 'copyPics',
            type => 'yesno',
            label => 'CopyPictures',
            default => '1'
        }];
    }
    
    # Ignored for the moment
    sub wantsFieldsSelection
    {
        return 0;
    }
    sub getEndInfo
    {
        return "";
    }
    
    sub getItemsArray
    {
        my ($self, $file) = @_;
        
        my $loader = new GCItems($self->{options}->{parent});
        
        $loader->load($file, undef, undef, 1);
        $loader->{model} = $self->{options}->{parent}->{model};
        $self->{modelAlreadySet} = 1;
        $self->{extractedModel} = $loader->{model}->getName;

        if ($self->{options}->{copyPics})
        {
            #Copying pictures
            foreach (@{$loader->{itemArray}})
            {
                my $title = $_->{$loader->{model}->{commonFields}->{title}};
                foreach my $field(@{$loader->{model}->{fieldsImage}})
                {
                    (my $suffix = $_->{$field}) =~ s/.*?(\.[^.]*)$/$1/;
                    my $imageFile = $self->{options}->{parent}->getUniqueImageFileName($suffix, $title);
                    copy($_->{$field}, $imageFile)
                        if $_->{$field};
                    $_->{$field} = $imageFile;
                }
            }
        }
        
        return $loader->{itemArray};
    }
}

1;
